@@ -123,8 +123,6 @@ std::string APT::Solver::Clause::toString(pkgCache &cache, bool pretty, bool sho
 std::string Solver::Work::toString(pkgCache &cache) const
 {
    std::ostringstream out;
-   if (erased)
-      out << "Erased ";
    if (clause->optional)
       out << "Optional ";
    out << "Item (" << ssize_t(size <= clause->solutions.size() ? size : -1) << "@" << level << ") ";
@@ -382,67 +380,79 @@ bool Solver::Propagate()
    {
       Var var = propQ.front();
       propQ.pop();
-      if (value(var) == LiftedBool::True)
+
+      // Stale propagation. These do not happen in MiniSat, because it clears the propQ on
+      // conflict, however, we simply keep the item in here and skip it.
+      if (value(var) == LiftedBool::Undefined)
+	 continue;
+
+      Lit lit = value(var) == LiftedBool::False ? ~var : var;
+      if (not lit.sign())
+	 Discover(lit.var());
+
+      for (auto clause : watches(lit))
       {
-	 Discover(var);
-	 for (auto &clause : (*this)[var].clauses)
-	    if (not AddWork(Work{clause.get(), decisionLevel()}))
-	       return false;
-	 for (auto rclause : (*this)[var].rclauses)
-	 {
-	    if (not rclause->negative || rclause->optional || rclause->reason.empty())
-	       continue;
-	    if (unlikely(debug >= 3))
-	       std::cerr << "Propagate " << var.toString(cache) << " to NOT " << rclause->reason.toString(cache) << " for dep " << const_cast<Clause *>(rclause)->toString(cache) << std::endl;
-	    if (not Enqueue(~rclause->reason, rclause))
-	       return false;
-	 }
-      }
-      else if (value(var) == LiftedBool::False)
-      {
-	 for (auto rclause : (*this)[var].rclauses)
-	 {
-	    if (rclause->negative || rclause->reason.empty())
-	       continue;
-	    if (value(rclause->reason) == LiftedBool::False)
-	       continue;
-
-	    auto count = std::count_if(rclause->solutions.begin(), rclause->solutions.end(), [this](auto var)
-				       { return value(var) != LiftedBool::False; });
-
-	    if (count == 1 && value(rclause->reason) == LiftedBool::True)
-	    {
-	       if (unlikely(debug >= 3))
-		  std::cerr << "Propagate NOT " << var.toString(cache) << " to unit clause " << rclause->toString(cache);
-	       if (rclause->optional)
-	       {
-		  // Enqueue duplicated item, this will ensure we see it at the correct time
-		  if (not AddWork(Work{rclause, decisionLevel()}))
-		     return false;
-	       }
-	       else
-	       {
-		  // Find the variable that must be chosen and enqueue it as a fact
-		  for (auto sol : rclause->solutions)
-		     if (value(sol) == LiftedBool::Undefined && not Enqueue(sol, rclause))
-			return false;
-	       }
-	       continue;
-	    }
-	    if (count >= 1 || rclause->optional)
-	       continue;
-
-	    if (unlikely(debug >= 3))
-	       std::cerr << "Propagate NOT " << var.toString(cache) << " to " << rclause->reason.toString(cache) << " for dep " << const_cast<Clause *>(rclause)->toString(cache) << std::endl;
-
-	    if (not Enqueue(~rclause->reason, rclause)) // Last version invalidated
-	       return false;
-	 }
+	 if (Propagate(clause, lit))
+	    continue;
+	 return false;
       }
    }
    return true;
 }
 
+bool Solver::Propagate(const Clause *clause, Lit p)
+{
+   if (debug >= 3)
+      std::cerr << "Propagate " << p.toString(cache) << " to " << clause->toString(cache) << std::endl;
+   // Negative clauses are trivial
+   if (clause->negative)
+   {
+      // One of the solutions was set, so reject the reason
+      if (p != clause->reason)
+	 return Enqueue(~clause->reason, clause);
+
+      // Here we need to reject all conflicting solutions (reason -> none of solutions)
+      for (auto sol : clause->solutions)
+	 if (not Enqueue(~sol, clause))
+	    return false;
+
+      return true;
+   }
+
+   // Check if the clause is unit, conflict, or undecided
+   Lit unit;
+   for (auto sol : clause->solutions)
+   {
+      if (value(sol) == LiftedBool::False)
+	 continue;
+      if (not unit.empty() || clause->optional)
+      {
+	 // We found a second solution, so we are undecided
+	 if (p == clause->reason)
+	    return AddWork(Work{clause, decisionLevel()});
+	 return true;
+      }
+
+      unit = sol;
+   }
+
+   // The clause is now either unit or conflict
+   if (not unit.empty())
+   {
+      // Unit clause. If it is an active clause, enqueue the unit literal.
+      if (value(clause->reason) == LiftedBool::True)
+         return Enqueue(unit, clause);
+      return true;
+   }
+   else
+   {
+      // Conflict clause. If this clause is non-optional; reject it's "reason".
+      if (not clause->optional)
+	 return Enqueue(~clause->reason, clause);
+      return true;
+   }
+}
+
 void Solver::UndoOne()
 {
    auto trailItem = trail.back();
@@ -506,11 +516,9 @@ bool Solver::Pop()
       UndoOne();
 
    // We need to remove any work that is at a higher level.
-   // FIXME: We should just mark the entries as erased and only do a compaction
-   //        of the heap once we have a lot of erased entries in it.
    trailLim.pop_back();
    work.erase(std::remove_if(work.begin(), work.end(), [this](Work &w) -> bool
-			     { return w.level > decisionLevel() || w.erased; }),
+			     { return w.level > decisionLevel(); }),
 	      work.end());
    std::make_heap(work.begin(), work.end());
 
@@ -572,9 +580,6 @@ bool Solver::Solve()
       auto item = work.front();
       std::pop_heap(work.begin(), work.end());
       work.pop_back();
-      // This item has been replaced with a new one. Remove it.
-      if (item.erased)
-	 continue;
 
       if (std::any_of(item.clause->solutions.begin(), item.clause->solutions.end(), [this](auto ver)
 		      { return value(ver) == LiftedBool::True; }))
@@ -598,6 +603,7 @@ bool Solver::Solve()
       }
       else
       {
+	 abort();
 	 if (unlikely(debug >= 1))
 	    std::cerr << item.toString(cache) << "\n";
 	 // Enqueue produces the right error message for us here, given that reason has been assigned true already...
@@ -859,8 +865,11 @@ const Clause *DependencySolver::RegisterClause(Clause &&clause)
 
    clauses.push_back(std::make_unique<Clause>(std::move(clause)));
    auto const &inserted = clauses.back();
-   for (auto var : inserted->solutions)
-      (*this)[var].rclauses.push_back(inserted.get());
+   // Insert all our watches
+   if (not inserted->reason.empty())
+      watches(inserted->reason).push_back(inserted.get());
+   for (auto sol : inserted->solutions)
+      watches(inserted->negative ? sol : ~sol).push_back(inserted.get());
    return inserted.get();
 }
 
@@ -885,7 +894,7 @@ bool DependencySolver::ObsoletedByNewerSourceVersion(pkgCache::VerIterator cand)
       if (priority == 0 || priority < candPriority)
 	 continue;
 
-      pkgObsolete[pkg] = 2;
+      pkgObsolete[pkg] = LiftedBool::True;
       if (debug >= 3)
 	 std::cerr << "Obsolete: " << cand.ParentPkg().FullName() << "=" << cand.VerStr() << " due to " << ver.ParentPkg().FullName() << "=" << ver.VerStr() << "\n";
       return true;
@@ -898,8 +907,8 @@ bool DependencySolver::Obsolete(pkgCache::PkgIterator pkg, bool AllowManual) con
 {
    if ((*this)[pkg].flags.manual && not AllowManual)
       return false;
-   if (pkgObsolete[pkg] != 0)
-      return pkgObsolete[pkg] == 2;
+   if (pkgObsolete[pkg] != LiftedBool::Undefined)
+      return pkgObsolete[pkg] == LiftedBool::True;
 
    auto ver = GetCandidateVer(pkg);
 
@@ -909,7 +918,7 @@ bool DependencySolver::Obsolete(pkgCache::PkgIterator pkg, bool AllowManual) con
    {
       if (debug >= 3)
 	 std::cerr << "Obsolete: " << pkg.FullName() << " - not installable\n";
-      pkgObsolete[pkg] = 2;
+      pkgObsolete[pkg] = LiftedBool::True;
       return true;
    }
 
@@ -921,14 +930,14 @@ bool DependencySolver::Obsolete(pkgCache::PkgIterator pkg, bool AllowManual) con
    {
       if (ver.Downloadable())
       {
-	 pkgObsolete[pkg] = 1;
+	 pkgObsolete[pkg] = LiftedBool::False;
 	 return false;
       }
    }
 
    if (debug >= 3)
       std::cerr << "Obsolete: " << ver.ParentPkg().FullName() << "=" << ver.VerStr() << " - not installable\n";
-   pkgObsolete[pkg] = 2;
+   pkgObsolete[pkg] = LiftedBool::True;
    return true;
 }
 void DependencySolver::Discover(Var var)
@@ -1372,19 +1381,34 @@ bool DependencySolver::ToDepCache(pkgDepCache &depcache) const
 std::string DependencySolver::InternalCliWhy(pkgDepCache &cache, pkgCache::PkgIterator pkg, bool assignment)
 {
    DependencySolver solver(cache.GetCache(), cache.GetPolicy(), EDSP::Request::Flags(0));
-   // In case nothing has a positive dependency on pkg it may not actually be discovered in a `why-not`
-   // scenario, so make sure we discover it explicitly.
-   solver.Discover(Var(pkg));
-   if (not solver.FromDepCache(cache) || not solver.Solve())
-      return "";
-   std::unordered_set<Var> seen;
    std::string buf;
-   if (solver[Var(pkg)].assignment == LiftedBool::Undefined)
-      return strprintf(buf, "%s is undecided\n", pkg.FullName().c_str()), buf;
-   if (assignment && solver[Var(pkg)].assignment != LiftedBool::True)
-      return strprintf(buf, "%s is not actually marked for install\n", pkg.FullName().c_str()), buf;
-   if (not assignment && solver[Var(pkg)].assignment == LiftedBool::True)
-      return strprintf(buf, "%s is actually marked for install\n", pkg.FullName().c_str()), buf;
-   return solver.LongWhyStr(Var(pkg), assignment, solver[Var(pkg)].reason, "", seen);
+
+   auto doOne = [&](Var var) -> std::string
+   {
+      solver.Discover(var);
+      if (not solver.FromDepCache(cache) || not solver.Solve())
+	 return "";
+      std::unordered_set<Var> seen;
+      if (solver[var].assignment == LiftedBool::Undefined)
+	 return strprintf(buf, "%s is undecided\n", var.toString(cache).c_str()), buf;
+      if (assignment && solver[var].assignment != LiftedBool::True)
+	 return strprintf(buf, "%s is not actually marked for install\n", var.toString(cache).c_str()), buf;
+      if (not assignment && solver[var].assignment == LiftedBool::True)
+	 return strprintf(buf, "%s is actually marked for install\n", var.toString(cache).c_str()), buf;
+      return solver.LongWhyStr(var, assignment, solver[var].reason, "", seen);
+   };
+
+   if (pkg.VersionList().end())
+   {
+      if (pkg.ProvidesList().end())
+	 return strprintf(buf, "%s is a virtual package without a provider\n", pkg.FullName().c_str()), buf;
+
+      std::ostringstream out;
+      ioprintf(out, "%s is a virtual package, printing all providers\n", pkg.FullName().c_str());
+      for (auto prv = pkg.ProvidesList(); not prv.end(); ++prv)
+	 out << doOne(Var(prv.OwnerVer()));
+      return out.str();
+   }
+   return doOne(Var(pkg));
 }
 } // namespace APT::Solver
